'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = initAppManager;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _config = require('./utils/config');

var _shimHistory = require('./utils/shim-history');

var _shimHistory2 = _interopRequireDefault(_shimHistory);

var _reconcile = require('./utils/reconcile');

var _reconcile2 = _interopRequireDefault(_reconcile);

var _lifecycle = require('./utils/lifecycle');

var _lifecycle2 = _interopRequireDefault(_lifecycle);

var _getState = require('./utils/get-state');

var _getState2 = _interopRequireDefault(_getState);

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initAppManager(w) {
  if (!w.document || !w.location || !w.history) {
    throw new _appManagerError2.default('Invalid window object. Aborting.', {
      source: 'init',
      level: _constants.levels.ERROR,
      fragment: null,
      slot: null,
      app: null,
      path: null,
      code: 'invalid_environment',
      recoverable: false
    });
  }

  function selectFromDom(querySelector) {
    return w.document.querySelector(querySelector);
  }

  function getBrowserLocation() {
    return w.location;
  }

  function appManager(config, events, overrideOptions) {
    var getSlotElement = function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slotName, fragmentName) {
        var errData, slot, querySelector, element;
        return _regenerator2.default.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                errData = {
                  source: 'get_slot_element',
                  fragment: fragmentName,
                  slot: slotName,
                  app: state ? state.app.name : null,
                  path: state ? state.path : null
                };
                slot = slots[slotName];

                if (slot) {
                  _context.next = 4;
                  break;
                }

                throw new _appManagerError2.default('Slot ' + slotName + ' could not be found.', (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'slot_not_found',
                  recoverable: false
                }));

              case 4:
                querySelector = slot.querySelector;

                if (querySelector) {
                  _context.next = 7;
                  break;
                }

                return _context.abrupt('return', null);

              case 7:
                _context.next = 9;
                return (0, _config.retry)(function () {
                  return selectFromDom(querySelector);
                }, _constants.DOM_CHECK_INTERVAL, options.importTimeout);

              case 9:
                element = _context.sent;

                if (element) {
                  _context.next = 12;
                  break;
                }

                throw new _appManagerError2.default('Element ' + querySelector + ' not found for slot ' + slotName + '.', (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'element_not_found',
                  recoverable: false
                }));

              case 12:
                return _context.abrupt('return', element);

              case 13:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this);
      }));

      return function getSlotElement(_x, _x2) {
        return _ref.apply(this, arguments);
      };
    }();

    if (!events || typeof events.emit !== 'function' || typeof events.on !== 'function' || typeof events.removeListener !== 'function') {
      throw new _appManagerError2.default('Invalid events object. Aborting.', {
        source: 'init',
        level: _constants.levels.ERROR,
        fragment: null,
        slot: null,
        app: null,
        path: null,
        code: 'invalid_events',
        recoverable: false
      });
    }

    var runningStateChange = null;
    var state = void 0;

    var queuedStateChange = null;

    var defaultOptions = {
      haltOnError: false,
      importTimeout: _constants.DEFAULT_TIMEOUT,
      getAppNameFromUrl: (0, _config.defaultGetAppNameFromUrl)(config.apps),
      onError: function onError(err) {
        events.emit(_constants.eventTitles.ERROR, err);
        events.emit(_constants.eventTitles.STATUS_CHANGE, (0, _config.getErrorStatusDetails)(err));
      }
    };

    var options = (0, _assign2.default)({}, defaultOptions, overrideOptions || {});

    var slots = (0, _config.setNames)(config.slots);
    var fragments = (0, _config.setNames)(config.fragments);
    var apps = (0, _config.setNames)(config.apps);

    var reconcile = (0, _reconcile2.default)(config);
    var lifecycle = (0, _lifecycle2.default)(w, fragments, getSlotElement, options);
    var getState = (0, _getState2.default)(apps, options);

    function getStateChanger(initialRender, eventTitle) {
      return function () {
        var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
          var currentApp, _getBrowserLocation, _pathname, _search, newState, pageDiff, haveScripts, nextStateChange;

          return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  _context2.prev = 0;
                  currentApp = state ? state.app : null;
                  _getBrowserLocation = getBrowserLocation(), _pathname = _getBrowserLocation.pathname, _search = _getBrowserLocation.search;
                  _context2.next = 5;
                  return getState('' + _pathname + _search, currentApp, eventTitle);

                case 5:
                  newState = _context2.sent;

                  if (newState) {
                    _context2.next = 9;
                    break;
                  }

                  events.emit(_constants.eventTitles.EXTERNAL_LINK);
                  return _context2.abrupt('return');

                case 9:

                  state = newState;
                  _context2.next = 16;
                  break;

                case 12:
                  _context2.prev = 12;
                  _context2.t0 = _context2['catch'](0);

                  options.onError(_context2.t0);
                  return _context2.abrupt('return');

                case 16:

                  events.emit(_constants.eventTitles.STATUS_CHANGE, (0, _config.getLoadingStatusDetails)());

                  _context2.prev = 17;
                  pageDiff = reconcile(state, initialRender);
                  _context2.next = 21;
                  return lifecycle(state, pageDiff, initialRender);

                case 21:
                  haveScripts = _context2.sent;


                  events.emit(_constants.eventTitles.STATUS_CHANGE, (0, _config.getDefaultStatusDetails)());

                  if (haveScripts) {
                    _context2.next = 26;
                    break;
                  }

                  events.emit(_constants.eventTitles.EXTERNAL_LINK);
                  return _context2.abrupt('return');

                case 26:
                  _context2.next = 31;
                  break;

                case 28:
                  _context2.prev = 28;
                  _context2.t1 = _context2['catch'](17);

                  options.onError(_context2.t1);

                case 31:
                  nextStateChange = queuedStateChange;


                  if (typeof nextStateChange === 'function' && !options.haltOnError) {
                    queuedStateChange = null;
                    runningStateChange = nextStateChange();
                  } else {
                    runningStateChange = null;
                  }

                case 33:
                case 'end':
                  return _context2.stop();
              }
            }
          }, _callee2, this, [[0, 12], [17, 28]]);
        }));

        function stateChanger() {
          return _ref2.apply(this, arguments);
        }

        return stateChanger;
      }();
    }

    function handleStateChange(eventTitle) {
      var stateChanger = getStateChanger(false, eventTitle);

      if (runningStateChange) {
        queuedStateChange = stateChanger;
      } else {
        runningStateChange = stateChanger();
      }
    }

    (0, _shimHistory2.default)(w, events, handleStateChange);

    runningStateChange = getStateChanger(true)();

    return {
      getState: function getState() {
        return state;
      },
      getRunningStateChange: function getRunningStateChange() {
        return runningStateChange;
      }
    };
  }

  appManager.statuses = _constants.statuses;
  appManager.eventTitles = _constants.eventTitles;
  appManager.levels = _constants.levels;

  return appManager;
}