'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = appManager;

var _appManagerError = require('./utils/app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _config = require('./utils/config');

var _constants = require('./constants');

var _reconcile2 = require('./utils/reconcile');

var _reconcile3 = _interopRequireDefault(_reconcile2);

var _getState = require('./utils/get-state');

var _getState2 = _interopRequireDefault(_getState);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function appManager(config, overrideOptions) {
  var apps = (0, _config.setNames)(config.apps);
  var fragments = (0, _config.setNames)(config.fragments);

  var defaultOptions = {
    getAppNameFromUrl: (0, _config.defaultGetAppNameFromUrl)(apps)
  };

  var options = (0, _assign2.default)({}, defaultOptions, overrideOptions || {});
  var reconcile = (0, _reconcile3.default)(config);
  var getState = (0, _getState2.default)(apps, options);

  return {
    getSlotsMarkup: function () {
      var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(url) {
        var _this = this;

        for (var _len = arguments.length, getMarkupArgs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
          getMarkupArgs[_key - 1] = arguments[_key];
        }

        var currentAppName, _url$split, _url$split2, path, errData, state, _reconcile, mount, getMarkupPromises, resolvedMarkup;

        return _regenerator2.default.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                currentAppName = null;
                _url$split = url.split('?'), _url$split2 = (0, _slicedToArray3.default)(_url$split, 1), path = _url$split2[0];
                errData = {
                  source: 'get_slots_markup',
                  fragment: null,
                  slot: null,
                  app: currentAppName,
                  path: path
                };

                if (!(typeof options.getAppNameFromUrl === 'function')) {
                  _context2.next = 13;
                  break;
                }

                _context2.prev = 4;
                _context2.next = 7;
                return options.getAppNameFromUrl(url);

              case 7:
                currentAppName = _context2.sent;
                _context2.next = 13;
                break;

              case 10:
                _context2.prev = 10;
                _context2.t0 = _context2['catch'](4);
                throw new _appManagerError2.default(_context2.t0, (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'get_app_name_from_location',
                  recoverable: false
                }));

              case 13:
                if (currentAppName) {
                  _context2.next = 15;
                  break;
                }

                throw new _appManagerError2.default('Could not find a route to match path', (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'invalid_route',
                  recoverable: false
                }));

              case 15:
                _context2.next = 17;
                return getState(url);

              case 17:
                state = _context2.sent;

                if (state) {
                  _context2.next = 20;
                  break;
                }

                throw new _appManagerError2.default('Could not get app state', (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'get_state',
                  recoverable: false
                }));

              case 20:
                _reconcile = reconcile(state, true), mount = _reconcile.mount;

                getMarkupPromises = function getMarkupPromises() {
                  return (0, _keys2.default)(mount).map(function () {
                    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slotName) {
                      var markup, fragmentName, fragment;
                      return _regenerator2.default.wrap(function _callee$(_context) {
                        while (1) {
                          switch (_context.prev = _context.next) {
                            case 0:
                              markup = '';
                              fragmentName = mount[slotName];

                              if (!fragmentName) {
                                _context.next = 8;
                                break;
                              }

                              fragment = fragments[fragmentName];

                              if (!(typeof fragment.getMarkup === 'function')) {
                                _context.next = 8;
                                break;
                              }

                              _context.next = 7;
                              return fragment.getMarkup.apply(fragment, [state].concat((0, _toConsumableArray3.default)(getMarkupArgs)));

                            case 7:
                              markup = _context.sent;

                            case 8:
                              return _context.abrupt('return', { slotName: slotName, markup: markup });

                            case 9:
                            case 'end':
                              return _context.stop();
                          }
                        }
                      }, _callee, _this);
                    }));

                    return function (_x2) {
                      return _ref2.apply(this, arguments);
                    };
                  }());
                };

                resolvedMarkup = void 0;
                _context2.prev = 23;
                _context2.next = 26;
                return _promise2.default.all(getMarkupPromises());

              case 26:
                resolvedMarkup = _context2.sent;
                _context2.next = 32;
                break;

              case 29:
                _context2.prev = 29;
                _context2.t1 = _context2['catch'](23);
                throw new _appManagerError2.default(_context2.t1, (0, _assign2.default)({}, errData, {
                  level: _constants.levels.ERROR,
                  code: 'get_markup',
                  recoverable: false
                }));

              case 32:
                return _context2.abrupt('return', resolvedMarkup.reduce(function (acc, slotMarkup) {
                  return slotMarkup ? (0, _assign2.default)({}, acc, (0, _defineProperty3.default)({}, slotMarkup.slotName, slotMarkup.markup)) : acc;
                }, {}));

              case 33:
              case 'end':
                return _context2.stop();
            }
          }
        }, _callee2, this, [[4, 10], [23, 29]]);
      }));

      function getSlotsMarkup(_x) {
        return _ref.apply(this, arguments);
      }

      return getSlotsMarkup;
    }()
  };
}