'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _asyncGenerator2 = require('babel-runtime/helpers/asyncGenerator');

var _asyncGenerator3 = _interopRequireDefault(_asyncGenerator2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

exports.default = initGetLifecycle;

var _config = require('./config');

var _appManagerError = require('./app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initGetLifecycle(w, fragments, getSlotElement, options) {
  var loadScript = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(state, slotName, fragmentName) {
      var errData, script, fragment;
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              errData = {
                source: 'load_script',
                fragment: fragmentName,
                slot: slotName,
                app: state ? state.app.name : null,
                path: state ? state.path : null
              };
              script = void 0;
              fragment = fragments[fragmentName];

              if (fragment) {
                _context.next = 5;
                break;
              }

              throw new _appManagerError2.default('Fragment ' + fragmentName + ' could not be found', (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'missing_fragment',
                recoverable: true
              }));

            case 5:
              if (!(typeof fragment.loadScript !== 'function')) {
                _context.next = 7;
                break;
              }

              return _context.abrupt('return', null);

            case 7:
              _context.prev = 7;
              _context.next = 10;
              return _promise2.default.race([fragment.loadScript(state), (0, _config.delay)(options.importTimeout)]);

            case 10:
              script = _context.sent;
              _context.next = 16;
              break;

            case 13:
              _context.prev = 13;
              _context.t0 = _context['catch'](7);
              throw new _appManagerError2.default(_context.t0, (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'load_script',
                recoverable: true
              }));

            case 16:
              if (script) {
                _context.next = 18;
                break;
              }

              throw new _appManagerError2.default('loadScript timed out for fragment ' + fragmentName + '.', (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'time_out',
                recoverable: true
              }));

            case 18:
              if ('version' in script) {
                _context.next = 20;
                break;
              }

              throw new _appManagerError2.default('Loaded script for fragment ' + fragmentName + ' does not have a version property.', (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'invalid_script',
                recoverable: true
              }));

            case 20:
              return _context.abrupt('return', script);

            case 21:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, this, [[7, 13]]);
    }));

    return function loadScript(_x, _x2, _x3) {
      return _ref.apply(this, arguments);
    };
  }();

  var hydrateScript = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(state, script, slotName, fragmentName) {
      var slotElement, errData;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return getSlotElement(slotName, fragmentName);

            case 2:
              slotElement = _context2.sent;
              errData = {
                source: 'hydrate',
                fragment: fragmentName,
                slot: slotName,
                app: state ? state.app.name : null,
                path: state ? state.path : null
              };

              if (slotElement) {
                _context2.next = 6;
                break;
              }

              return _context2.abrupt('return');

            case 6:
              _context2.prev = 6;

              if (!(script.version === 3)) {
                _context2.next = 12;
                break;
              }

              _context2.next = 10;
              return script.hydrate(slotElement, w.history, state.app);

            case 10:
              _context2.next = 30;
              break;

            case 12:
              if (!(script.version === 4)) {
                _context2.next = 22;
                break;
              }

              if (!(typeof script.hydrate === 'function')) {
                _context2.next = 18;
                break;
              }

              _context2.next = 16;
              return script.hydrate(slotElement, state.app);

            case 16:
              _context2.next = 20;
              break;

            case 18:
              _context2.next = 20;
              return script.mount(slotElement, _constants.eventTitles.HISTORY_REPLACE_STATE, state.app);

            case 20:
              _context2.next = 30;
              break;

            case 22:
              if (!(typeof script.hydrate === 'function')) {
                _context2.next = 27;
                break;
              }

              _context2.next = 25;
              return script.hydrate(slotElement, state);

            case 25:
              _context2.next = 30;
              break;

            case 27:
              if (!(typeof script.render === 'function')) {
                _context2.next = 30;
                break;
              }

              _context2.next = 30;
              return script.render(slotElement, state);

            case 30:
              _context2.next = 35;
              break;

            case 32:
              _context2.prev = 32;
              _context2.t0 = _context2['catch'](6);
              throw new _appManagerError2.default(_context2.t0, (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'hydrate',
                recoverable: true
              }));

            case 35:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, this, [[6, 32]]);
    }));

    return function hydrateScript(_x4, _x5, _x6, _x7) {
      return _ref2.apply(this, arguments);
    };
  }();

  var renderScript = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(state, script, slotName, fragmentName) {
      var slotElement, errData;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return getSlotElement(slotName, fragmentName);

            case 2:
              slotElement = _context3.sent;
              errData = {
                source: 'render',
                fragment: fragmentName,
                slot: slotName,
                app: state ? state.app.name : null,
                path: state ? state.path : null
              };

              if (slotElement) {
                _context3.next = 6;
                break;
              }

              return _context3.abrupt('return');

            case 6:
              _context3.prev = 6;

              if (!(script.version === 3)) {
                _context3.next = 12;
                break;
              }

              _context3.next = 10;
              return script.mount(slotElement, w.history, state.app);

            case 10:
              _context3.next = 21;
              break;

            case 12:
              if (!(script.version === 4)) {
                _context3.next = 18;
                break;
              }

              if (!state.event) {
                _context3.next = 16;
                break;
              }

              _context3.next = 16;
              return script.mount(slotElement, state.event, state.app);

            case 16:
              _context3.next = 21;
              break;

            case 18:
              if (!(typeof script.render === 'function')) {
                _context3.next = 21;
                break;
              }

              _context3.next = 21;
              return script.render(slotElement, state);

            case 21:
              _context3.next = 26;
              break;

            case 23:
              _context3.prev = 23;
              _context3.t0 = _context3['catch'](6);
              throw new _appManagerError2.default(_context3.t0, (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'render',
                recoverable: true
              }));

            case 26:
            case 'end':
              return _context3.stop();
          }
        }
      }, _callee3, this, [[6, 23]]);
    }));

    return function renderScript(_x8, _x9, _x10, _x11) {
      return _ref3.apply(this, arguments);
    };
  }();

  var updateScript = function () {
    var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(state, script, slotName, fragmentName) {
      var errData;
      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              errData = {
                source: 'on_state_change',
                fragment: fragmentName,
                slot: slotName,
                app: state ? state.app.name : null,
                path: state ? state.path : null
              };
              _context4.prev = 1;

              if (!(script.version === 3)) {
                _context4.next = 7;
                break;
              }

              _context4.next = 5;
              return script.onStateChange(w.history, state.app);

            case 5:
              _context4.next = 17;
              break;

            case 7:
              if (!(script.version === 4)) {
                _context4.next = 14;
                break;
              }

              if (!(typeof script.onStateChange === 'function')) {
                _context4.next = 12;
                break;
              }

              if (!state.event) {
                _context4.next = 12;
                break;
              }

              _context4.next = 12;
              return script.onStateChange(state.event, state.app);

            case 12:
              _context4.next = 17;
              break;

            case 14:
              if (!(typeof script.onStateChange === 'function')) {
                _context4.next = 17;
                break;
              }

              _context4.next = 17;
              return script.onStateChange(state);

            case 17:
              _context4.next = 22;
              break;

            case 19:
              _context4.prev = 19;
              _context4.t0 = _context4['catch'](1);
              throw new _appManagerError2.default(_context4.t0, (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'on_state_change',
                recoverable: true
              }));

            case 22:
            case 'end':
              return _context4.stop();
          }
        }
      }, _callee4, this, [[1, 19]]);
    }));

    return function updateScript(_x12, _x13, _x14, _x15) {
      return _ref4.apply(this, arguments);
    };
  }();

  var unmountScript = function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(state, script, slotName, fragmentName) {
      var errData, slotElement;
      return _regenerator2.default.wrap(function _callee5$(_context5) {
        while (1) {
          switch (_context5.prev = _context5.next) {
            case 0:
              errData = {
                source: 'unmount',
                fragment: fragmentName,
                slot: slotName,
                app: state ? state.app.name : null,
                path: state ? state.path : null
              };
              _context5.next = 3;
              return getSlotElement(slotName, fragmentName);

            case 3:
              slotElement = _context5.sent;

              if (slotElement) {
                _context5.next = 6;
                break;
              }

              return _context5.abrupt('return');

            case 6:
              if (state.prevApp) {
                _context5.next = 8;
                break;
              }

              throw new _appManagerError2.default('Invalid state at unmount for fragment ' + fragmentName, (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'invalid_state',
                recoverable: true
              }));

            case 8:
              _context5.prev = 8;

              if (!(script.version === 3)) {
                _context5.next = 14;
                break;
              }

              _context5.next = 12;
              return script.unmount(slotElement, w.history, state.prevApp);

            case 12:
              _context5.next = 23;
              break;

            case 14:
              if (!(script.version === 4)) {
                _context5.next = 20;
                break;
              }

              if (!state.event) {
                _context5.next = 18;
                break;
              }

              _context5.next = 18;
              return script.unmount(slotElement, state.event, state.prevApp);

            case 18:
              _context5.next = 23;
              break;

            case 20:
              if (!(typeof script.unmount === 'function')) {
                _context5.next = 23;
                break;
              }

              _context5.next = 23;
              return script.unmount(slotElement, state);

            case 23:
              _context5.next = 28;
              break;

            case 25:
              _context5.prev = 25;
              _context5.t0 = _context5['catch'](8);
              throw new _appManagerError2.default(_context5.t0, (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'unmount',
                recoverable: true
              }));

            case 28:
            case 'end':
              return _context5.stop();
          }
        }
      }, _callee5, this, [[8, 25]]);
    }));

    return function unmountScript(_x16, _x17, _x18, _x19) {
      return _ref5.apply(this, arguments);
    };
  }();

  var updateScriptStatus = function () {
    var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(state, script, statusDetails) {
      return _regenerator2.default.wrap(function _callee6$(_context6) {
        while (1) {
          switch (_context6.prev = _context6.next) {
            case 0:
              if (!(script.version === 3)) {
                _context6.next = 2;
                break;
              }

              return _context6.abrupt('return');

            case 2:
              _context6.prev = 2;

              if (!(script.version === 4)) {
                _context6.next = 9;
                break;
              }

              if (!(typeof script.onUpdateStatus === 'function')) {
                _context6.next = 7;
                break;
              }

              _context6.next = 7;
              return script.onUpdateStatus(statusDetails.status, state.app);

            case 7:
              _context6.next = 12;
              break;

            case 9:
              if (!(typeof script.onUpdateStatus === 'function')) {
                _context6.next = 12;
                break;
              }

              _context6.next = 12;
              return script.onUpdateStatus(statusDetails, state);

            case 12:
              _context6.next = 16;
              break;

            case 14:
              _context6.prev = 14;
              _context6.t0 = _context6['catch'](2);

            case 16:
            case 'end':
              return _context6.stop();
          }
        }
      }, _callee6, this, [[2, 14]]);
    }));

    return function updateScriptStatus(_x20, _x21, _x22) {
      return _ref6.apply(this, arguments);
    };
  }();

  var stateChanger = function () {
    var _ref7 = _asyncGenerator3.default.wrap( /*#__PURE__*/_regenerator2.default.mark(function _callee7(initialState, script, slotName, fragmentName) {
      var state;
      return _regenerator2.default.wrap(function _callee7$(_context7) {
        while (1) {
          switch (_context7.prev = _context7.next) {
            case 0:
              state = initialState;

            case 1:
              if (!true) {
                _context7.next = 30;
                break;
              }

            case 2:
              if (!true) {
                _context7.next = 16;
                break;
              }

              _context7.prev = 3;
              _context7.next = 6;
              return;

            case 6:
              state = _context7.sent;
              return _context7.abrupt('break', 16);

            case 10:
              _context7.prev = 10;
              _context7.t0 = _context7['catch'](3);
              _context7.next = 14;
              return _asyncGenerator3.default.await(updateScriptStatus(state, script, (0, _config.getErrorStatusDetails)(_context7.t0)));

            case 14:
              _context7.next = 2;
              break;

            case 16:
              _context7.next = 18;
              return _asyncGenerator3.default.await(updateScriptStatus(state, script, defaultStatusDetails));

            case 18:
              _context7.next = 20;
              return;

            case 20:
              state = _context7.sent;
              _context7.next = 23;
              return _asyncGenerator3.default.await(updateScriptStatus(state, script, loadingStatusDetails));

            case 23:
              _context7.next = 25;
              return;

            case 25:
              state = _context7.sent;
              _context7.next = 28;
              return _asyncGenerator3.default.await(updateScript(state, script, slotName, fragmentName));

            case 28:
              _context7.next = 1;
              break;

            case 30:
            case 'end':
              return _context7.stop();
          }
        }
      }, _callee7, this, [[3, 10]]);
    }));

    return function stateChanger(_x23, _x24, _x25, _x26) {
      return _ref7.apply(this, arguments);
    };
  }();

  var updateSlot = function () {
    var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(state, slotName, fragmentName) {
      var script, unmount, _ref9, done;

      return _regenerator2.default.wrap(function _callee8$(_context8) {
        while (1) {
          switch (_context8.prev = _context8.next) {
            case 0:
              script = cachedScripts[fragmentName];
              unmount = true;
              _context8.prev = 2;

              if (!mountedStateChangers[slotName]) {
                _context8.next = 12;
                break;
              }

              _context8.next = 6;
              return mountedStateChangers[slotName].next(state);

            case 6:
              _ref9 = _context8.sent;
              done = _ref9.done;

              unmount = done;

              if (!(unmount && script)) {
                _context8.next = 12;
                break;
              }

              _context8.next = 12;
              return updateScriptStatus(state, script, defaultStatusDetails);

            case 12:
              _context8.next = 20;
              break;

            case 14:
              _context8.prev = 14;
              _context8.t0 = _context8['catch'](2);

              if (!script) {
                _context8.next = 19;
                break;
              }

              _context8.next = 19;
              return updateScriptStatus(state, script, (0, _config.getErrorStatusDetails)(_context8.t0));

            case 19:
              throw _context8.t0;

            case 20:
              _context8.prev = 20;

              if (!unmount) {
                _context8.next = 26;
                break;
              }

              if (mountedStateChangers[slotName]) {
                delete mountedStateChangers[slotName];
              }

              if (!script) {
                _context8.next = 26;
                break;
              }

              _context8.next = 26;
              return unmountScript(state, script, slotName, fragmentName);

            case 26:
              return _context8.finish(20);

            case 27:
            case 'end':
              return _context8.stop();
          }
        }
      }, _callee8, this, [[2, 14, 20, 27]]);
    }));

    return function updateSlot(_x27, _x28, _x29) {
      return _ref8.apply(this, arguments);
    };
  }();

  var returnSlot = function () {
    var _ref10 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(state, slotName, fragmentName) {
      return _regenerator2.default.wrap(function _callee9$(_context9) {
        while (1) {
          switch (_context9.prev = _context9.next) {
            case 0:
              if (!mountedStateChangers[slotName]) {
                _context9.next = 3;
                break;
              }

              _context9.next = 3;
              return mountedStateChangers[slotName].return();

            case 3:
              _context9.next = 5;
              return updateSlot(state, slotName, fragmentName);

            case 5:
            case 'end':
              return _context9.stop();
          }
        }
      }, _callee9, this);
    }));

    return function returnSlot(_x30, _x31, _x32) {
      return _ref10.apply(this, arguments);
    };
  }();

  var mountSlot = function () {
    var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10(state, slotName, fragmentName, initialRender) {
      var app, path, errData, cachedScript, script;
      return _regenerator2.default.wrap(function _callee10$(_context10) {
        while (1) {
          switch (_context10.prev = _context10.next) {
            case 0:
              app = state.app, path = state.path;
              errData = {
                source: 'mount_slot',
                fragment: fragmentName || null,
                slot: slotName,
                app: app ? app.name : null,
                path: path || null
              };
              _context10.prev = 2;

              if (!mountedStateChangers[slotName]) {
                _context10.next = 5;
                break;
              }

              throw new _appManagerError2.default('Overwriting an existing fragment.', (0, _assign2.default)({}, errData, {
                level: _constants.levels.ERROR,
                code: 'overwrite_existing',
                recoverable: true
              }));

            case 5:
              cachedScript = cachedScripts[fragmentName];
              script = void 0;

              if (!cachedScript) {
                _context10.next = 11;
                break;
              }

              script = cachedScript;
              _context10.next = 17;
              break;

            case 11:
              _context10.next = 13;
              return loadScript(state, slotName, fragmentName);

            case 13:
              script = _context10.sent;

              if (script) {
                _context10.next = 16;
                break;
              }

              return _context10.abrupt('return', false);

            case 16:

              cachedScripts[fragmentName] = script;

            case 17:
              _context10.next = 19;
              return updateScriptStatus(state, script, loadingStatusDetails);

            case 19:
              _context10.prev = 19;

              if (!initialRender) {
                _context10.next = 25;
                break;
              }

              _context10.next = 23;
              return hydrateScript(state, script, slotName, fragmentName);

            case 23:
              _context10.next = 27;
              break;

            case 25:
              _context10.next = 27;
              return renderScript(state, script, slotName, fragmentName);

            case 27:

              mountedStateChangers[slotName] = stateChanger(state, script, slotName, fragmentName);
              _context10.next = 35;
              break;

            case 30:
              _context10.prev = 30;
              _context10.t0 = _context10['catch'](19);
              _context10.next = 34;
              return updateScriptStatus(state, script, (0, _config.getErrorStatusDetails)(_context10.t0));

            case 34:
              throw _context10.t0;

            case 35:
              _context10.prev = 35;
              _context10.next = 38;
              return updateSlot(state, slotName, fragmentName);

            case 38:
              return _context10.finish(35);

            case 39:
              return _context10.abrupt('return', true);

            case 40:
            case 'end':
              return _context10.stop();
          }
        }
      }, _callee10, this, [[2,, 35, 39], [19, 30]]);
    }));

    return function mountSlot(_x33, _x34, _x35, _x36) {
      return _ref11.apply(this, arguments);
    };
  }();

  var handleErrors = function () {
    var _ref12 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(err) {
      var errorPromises;
      return _regenerator2.default.wrap(function _callee11$(_context11) {
        while (1) {
          switch (_context11.prev = _context11.next) {
            case 0:
              errorPromises = function errorPromises() {
                return (0, _keys2.default)(mountedStateChangers).map(function (errorSlot) {
                  return mountedStateChangers[errorSlot].throw(err);
                });
              };

              _context11.next = 3;
              return _promise2.default.all(errorPromises());

            case 3:
            case 'end':
              return _context11.stop();
          }
        }
      }, _callee11, this);
    }));

    return function handleErrors(_x37) {
      return _ref12.apply(this, arguments);
    };
  }();

  var updateAllSlots = function () {
    var _ref13 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(state) {
      var updateAllSlotsPromises;
      return _regenerator2.default.wrap(function _callee12$(_context12) {
        while (1) {
          switch (_context12.prev = _context12.next) {
            case 0:
              updateAllSlotsPromises = function updateAllSlotsPromises() {
                return (0, _keys2.default)(mountedStateChangers).map(function (statusSlot) {
                  return mountedStateChangers[statusSlot].next(state);
                });
              };

              return _context12.abrupt('return', _promise2.default.all(updateAllSlotsPromises()));

            case 2:
            case 'end':
              return _context12.stop();
          }
        }
      }, _callee12, this);
    }));

    return function updateAllSlots(_x38) {
      return _ref13.apply(this, arguments);
    };
  }();

  var mountedStateChangers = {};
  var cachedScripts = {};

  var loadingStatusDetails = (0, _config.getLoadingStatusDetails)();
  var defaultStatusDetails = (0, _config.getDefaultStatusDetails)();

  return function () {
    var _ref14 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee16(state, pageDiff, initialRender) {
      var _this = this;

      var unmount, mount, update, onError, updatePromises, unmountPromises, mountPromises, haveScripts;
      return _regenerator2.default.wrap(function _callee16$(_context16) {
        while (1) {
          switch (_context16.prev = _context16.next) {
            case 0:
              unmount = pageDiff.unmount, mount = pageDiff.mount, update = pageDiff.update;
              onError = typeof options.onError === 'function' ? options.onError : function (_err) {};

              updatePromises = function updatePromises() {
                return (0, _keys2.default)(update).map(function () {
                  var _ref15 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(slotName) {
                    return _regenerator2.default.wrap(function _callee13$(_context13) {
                      while (1) {
                        switch (_context13.prev = _context13.next) {
                          case 0:
                            _context13.prev = 0;
                            _context13.next = 3;
                            return updateSlot(state, slotName, update[slotName]);

                          case 3:
                            _context13.next = 10;
                            break;

                          case 5:
                            _context13.prev = 5;
                            _context13.t0 = _context13['catch'](0);
                            _context13.next = 9;
                            return handleErrors(_context13.t0);

                          case 9:
                            onError(_context13.t0);

                          case 10:
                          case 'end':
                            return _context13.stop();
                        }
                      }
                    }, _callee13, _this, [[0, 5]]);
                  }));

                  return function (_x42) {
                    return _ref15.apply(this, arguments);
                  };
                }());
              };

              unmountPromises = function unmountPromises() {
                return (0, _keys2.default)(unmount).map(function () {
                  var _ref16 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14(slotName) {
                    return _regenerator2.default.wrap(function _callee14$(_context14) {
                      while (1) {
                        switch (_context14.prev = _context14.next) {
                          case 0:
                            _context14.prev = 0;
                            _context14.next = 3;
                            return returnSlot(state, slotName, unmount[slotName]);

                          case 3:
                            _context14.next = 10;
                            break;

                          case 5:
                            _context14.prev = 5;
                            _context14.t0 = _context14['catch'](0);
                            _context14.next = 9;
                            return handleErrors(_context14.t0);

                          case 9:
                            onError(_context14.t0);

                          case 10:
                          case 'end':
                            return _context14.stop();
                        }
                      }
                    }, _callee14, _this, [[0, 5]]);
                  }));

                  return function (_x43) {
                    return _ref16.apply(this, arguments);
                  };
                }());
              };

              mountPromises = function mountPromises() {
                return (0, _keys2.default)(mount).map(function () {
                  var _ref17 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee15(slotName) {
                    var hasScript;
                    return _regenerator2.default.wrap(function _callee15$(_context15) {
                      while (1) {
                        switch (_context15.prev = _context15.next) {
                          case 0:
                            hasScript = void 0;
                            _context15.prev = 1;
                            _context15.next = 4;
                            return mountSlot(state, slotName, mount[slotName], initialRender);

                          case 4:
                            hasScript = _context15.sent;
                            _context15.next = 13;
                            break;

                          case 7:
                            _context15.prev = 7;
                            _context15.t0 = _context15['catch'](1);
                            _context15.next = 11;
                            return handleErrors(_context15.t0);

                          case 11:
                            onError(_context15.t0);
                            hasScript = true;

                          case 13:
                            return _context15.abrupt('return', hasScript);

                          case 14:
                          case 'end':
                            return _context15.stop();
                        }
                      }
                    }, _callee15, _this, [[1, 7]]);
                  }));

                  return function (_x44) {
                    return _ref17.apply(this, arguments);
                  };
                }());
              };

              _context16.next = 7;
              return updateAllSlots(state);

            case 7:
              _context16.next = 9;
              return _promise2.default.all([].concat((0, _toConsumableArray3.default)(unmountPromises()), (0, _toConsumableArray3.default)(updatePromises())));

            case 9:
              _context16.next = 11;
              return _promise2.default.all(mountPromises());

            case 11:
              haveScripts = _context16.sent;
              _context16.next = 14;
              return updateAllSlots(state);

            case 14:
              return _context16.abrupt('return', haveScripts.every(Boolean));

            case 15:
            case 'end':
              return _context16.stop();
          }
        }
      }, _callee16, this);
    }));

    function updatePage(_x39, _x40, _x41) {
      return _ref14.apply(this, arguments);
    }

    return updatePage;
  }();
}