'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.default = initReconcile;

var _slotFinder = require('slot-finder');

var _slotFinder2 = _interopRequireDefault(_slotFinder);

var _config = require('./config');

var _appManagerError = require('./app-manager-error');

var _appManagerError2 = _interopRequireDefault(_appManagerError);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initReconcile(config) {
  var apps = (0, _config.setNames)(config.apps);
  var slots = (0, _config.setNames)(config.slots);
  var fragments = (0, _config.setNames)(config.fragments);

  function getPageFragments(appName, path) {
    var errData = {
      source: 'get_page_fragments',
      fragment: null,
      slot: null,
      app: appName,
      path: path
    };

    var app = apps[appName];

    var appFragmentNames = [];
    if (Array.isArray(app.fragments)) {
      appFragmentNames = app.fragments;
    } else if (typeof app.fragment === 'string') {
      appFragmentNames = [app.fragment];
    }

    var appFragments = appFragmentNames.map(function (fragmentName) {
      return fragments[fragmentName];
    });

    if (!appFragments.every(Boolean)) {
      throw new _appManagerError2.default('App ' + appName + ' tried to mount at least one missing fragment.', (0, _assign2.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'missing_fragment',
        recoverable: false
      }));
    }

    var fragmentsWithSlots = appFragments.map(function (fragment) {
      var fragmentSlots = [];
      if (Array.isArray(fragment.slots)) {
        fragmentSlots = fragment.slots;
      } else if (typeof fragment.slot === 'string') {
        fragmentSlots = [fragment.slot];
      }

      return { name: fragment.name, slots: fragmentSlots };
    });

    var emptySlots = fragmentsWithSlots.reduce(function (fragmentSlots, fragment) {
      return fragmentSlots.concat(fragment.slots);
    }, []).reduce(function (empty, slotName) {
      return (0, _assign2.default)(empty, (0, _defineProperty3.default)({}, slotName, null));
    }, {});

    var mountableSlots = (0, _keys2.default)(emptySlots).map(function (slotName) {
      return slots[slotName];
    });

    if (!mountableSlots.every(Boolean)) {
      throw new _appManagerError2.default('App ' + appName + ' tried to mount into at least one missing slot.', (0, _assign2.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'invalid_slots',
        recoverable: false
      }));
    }

    var pageFragments = void 0;

    try {
      pageFragments = (0, _slotFinder2.default)(emptySlots, fragmentsWithSlots);
    } catch (err) {
      throw new _appManagerError2.default(err, (0, _assign2.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'slot_finder',
        recoverable: true
      }));
    }

    if (!pageFragments || (typeof pageFragments === 'undefined' ? 'undefined' : (0, _typeof3.default)(pageFragments)) !== 'object') {
      throw new _appManagerError2.default('SlotFinder returned invalid fragments for app ' + appName, (0, _assign2.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'get_fragments',
        recoverable: true
      }));
    }

    return pageFragments;
  }

  return function reconcile(state, initialRender) {
    var app = state.app,
        path = state.path,
        prevApp = state.prevApp;


    var errData = {
      slot: null,
      fragment: null,
      source: 'reconcile',
      app: app ? app.name : null,
      path: path
    };

    if (!app) {
      throw new _appManagerError2.default('No app set when trying to reconcile page.', (0, _assign2.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'no_current_app',
        recoverable: true
      }));
    }

    var newAppFragments = getPageFragments(app.name, path);

    var newAppSlots = (0, _keys2.default)(newAppFragments);

    if (initialRender) {
      var _mount = newAppSlots.filter(function (slotName) {
        return newAppFragments[slotName];
      }).reduce(function (diff, slotName) {
        return (0, _assign2.default)({}, diff, (0, _defineProperty3.default)({}, slotName, newAppFragments[slotName]));
      }, {});

      return { mount: _mount, update: {}, unmount: {} };
    }

    if (!prevApp) {
      throw new _appManagerError2.default('No old app set when trying to mount ' + app.name, (0, _assign2.default)({}, errData, {
        level: _constants.levels.ERROR,
        code: 'no_previous_app',
        recoverable: false
      }));
    }

    var oldAppFragments = getPageFragments(prevApp.name, path);
    var oldAppSlots = (0, _keys2.default)(oldAppFragments);

    var update = oldAppSlots.filter(function (slotName) {
      var fragmentName = oldAppFragments[slotName];
      return fragmentName && newAppSlots.indexOf(slotName) > -1 && newAppFragments[slotName] === fragmentName;
    }).reduce(function (diff, slotName) {
      return (0, _assign2.default)({}, diff, (0, _defineProperty3.default)({}, slotName, newAppFragments[slotName]));
    }, {});

    var unmount = oldAppSlots.filter(function (slotName) {
      var fragmentName = oldAppFragments[slotName];
      return fragmentName && (newAppSlots.indexOf(slotName) === -1 || newAppFragments[slotName] !== fragmentName);
    }).reduce(function (diff, slotName) {
      return (0, _assign2.default)({}, diff, (0, _defineProperty3.default)({}, slotName, oldAppFragments[slotName]));
    }, {});

    var mount = newAppSlots.filter(function (slotName) {
      var fragmentName = newAppFragments[slotName];
      return fragmentName && (oldAppSlots.indexOf(slotName) === -1 || oldAppFragments[slotName] !== fragmentName);
    }).reduce(function (diff, slotName) {
      return (0, _assign2.default)({}, diff, (0, _defineProperty3.default)({}, slotName, newAppFragments[slotName]));
    }, {});

    return { unmount: unmount, mount: mount, update: update };
  };
}