'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = shimHistory;

var _constants = require('../constants');

function shimHistory(w, events, callback) {
  /* eslint-disable no-param-reassign */

  var onpopstate = w.onpopstate,
      onbeforeunload = w.onbeforeunload;
  var _w$history = w.history,
      pushState = _w$history.pushState,
      replaceState = _w$history.replaceState;


  w.history.pushState = function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (typeof pushState === 'function') {
      pushState.apply(w.history, args);
    }

    events.emit(_constants.eventTitles.HISTORY_PUSH_STATE);
    events.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_PUSH_STATE);
  };

  w.history.replaceState = function () {
    for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      args[_key2] = arguments[_key2];
    }

    if (typeof replaceState === 'function') {
      replaceState.apply(w.history, args);
    }

    events.emit(_constants.eventTitles.HISTORY_REPLACE_STATE);
    events.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_REPLACE_STATE);
  };

  // $FlowFixMe
  w.history.getState = function () {
    return {
      data: w.history.state,
      title: w.document.title,
      url: w.location.href,
      hash: '' + w.location.pathname + (w.location.search || '')
    };
  };

  w.onpopstate = function () {
    for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
      args[_key3] = arguments[_key3];
    }

    if (typeof onpopstate === 'function') {
      onpopstate.apply(w.history, args);
    }

    events.emit(_constants.eventTitles.HISTORY_POP_STATE);
    events.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_POP_STATE);
  };

  w.onbeforeunload = function () {
    for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
      args[_key4] = arguments[_key4];
    }

    if (typeof onbeforeunload === 'function') {
      onbeforeunload.apply(w.history, args);
    }

    events.removeListener(_constants.eventTitles.HISTORY_STATE_CHANGE, callback);

    w.onpopstate = onpopstate;
    w.history.pushState = pushState;
    w.history.replaceState = replaceState;
  };

  events.on(_constants.eventTitles.HISTORY_STATE_CHANGE, callback);
}