'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

var _eventemitter = require('eventemitter3');

var _eventemitter2 = _interopRequireDefault(_eventemitter);

var _urlParse = require('url-parse');

var _urlParse2 = _interopRequireDefault(_urlParse);

var _constants = require('../constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-line import/no-extraneous-dependencies
function urlToLocation(url) {
  var parsed = new _urlParse2.default(url);

  return {
    hash: parsed.hash,
    host: parsed.host,
    hostname: parsed.hostname,
    href: parsed.href,
    origin: parsed.origin,
    pathname: parsed.pathname,
    port: parsed.port,
    protocol: parsed.protocol,
    search: parsed.query
  };
} // eslint-disable-line import/no-extraneous-dependencies

var History = function () {
  function History(state, position, location, document, events) {
    var _this = this;

    (0, _classCallCheck3.default)(this, History);

    this.replaceState = function () {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var title = arguments[1];
      var href = arguments[2];
      var _state$_position = _this._state[_this._position],
          currentTitle = _state$_position.title,
          currentUrl = _state$_position.url;

      var newHash = href || currentUrl;
      var newTitle = title || currentTitle;

      _this._state = [].concat((0, _toConsumableArray3.default)(_this._state.slice(0, _this._position)), [{ data: data, title: newTitle, url: newHash }]);
      _this._updateLocation(newHash, newTitle);
    };

    this.pushState = function () {
      var data = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : null;
      var title = arguments[1];
      var href = arguments[2];
      var _state$_position2 = _this._state[_this._position],
          currentTitle = _state$_position2.title,
          currentUrl = _state$_position2.url;

      var newHash = href || currentUrl;
      var newTitle = title || currentTitle;

      _this._state = [].concat((0, _toConsumableArray3.default)(_this._state.slice(0, _this._position + 1)), [{ data: data, title: newTitle, url: newHash }]);
      _this._position += 1;
      _this._updateLocation(newHash, newTitle);
    };

    this.back = function () {
      _this._position -= 1;

      var newState = _this._state[_this._position];

      _this._updateLocation(newState.url, newState.title);
      _this._events.emit(_constants.eventTitles.HISTORY_POP_STATE, newState);
    };

    this.forward = function () {
      if (_this._state[_this._position + 1]) {
        _this._position += 1;

        var newState = _this._state[_this._position];

        _this._updateLocation(newState.url, newState.title);
        _this._events.emit(_constants.eventTitles.HISTORY_POP_STATE, newState);
      }
    };

    this.go = function (relativeIndex) {
      var newPosition = _this._position + relativeIndex;

      if (_this._state[newPosition]) {
        _this._position = newPosition;

        var newState = _this._state[_this._position];

        _this._updateLocation(newState.url, newState.title);
        _this._events.emit(_constants.eventTitles.HISTORY_POP_STATE, newState);
      }
    };

    this._state = state;
    this._position = position;
    this._events = events;
    this._location = location;
    this._document = document;
  }

  (0, _createClass3.default)(History, [{
    key: '_updateLocation',
    value: function _updateLocation(path, title) {
      (0, _assign2.default)(this._location, urlToLocation(path));
      (0, _assign2.default)(this._document, { title: title });
    }
  }, {
    key: 'length',
    get: function get() {
      return this._state.length;
    }
  }, {
    key: 'state',
    get: function get() {
      return (this._state[this._position] || {}).data;
    }
  }]);
  return History;
}();

var WindowStub = function () {
  function WindowStub() {
    var initialHistoryState = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [{ data: {}, title: null, url: '/' }];
    (0, _classCallCheck3.default)(this, WindowStub);
    this._events = new _eventemitter2.default();
    this._onpopstatecb = null;
    this._onbeforeunloadecb = null;

    var isInitialState = Array.isArray(initialHistoryState) && initialHistoryState.length > 0;

    this._position = isInitialState ? initialHistoryState.length - 1 : 0;
    this._state = [].concat((0, _toConsumableArray3.default)(initialHistoryState));

    var initialPath = isInitialState ? initialHistoryState[initialHistoryState.length - 1].url : '/';
    var initialTitle = isInitialState ? initialHistoryState[initialHistoryState.length - 1].title : null;

    var domDelay = null;

    this.location = urlToLocation(initialPath);
    this.document = {
      title: initialTitle,
      querySelector: function querySelector(selector) {
        // Arbitrarily delay the DOM element from being returned.
        setTimeout(function () {
          domDelay = selector;
        }, 15);
        var returnSelector = domDelay;
        domDelay = null;
        return returnSelector;
      }
    };

    this.history = new History(this._state, this._position, this.location, this.document, this._events);
  }

  (0, _createClass3.default)(WindowStub, [{
    key: 'onpopstate',
    set: function set(callback) {
      if (typeof callback !== 'function') {
        this._events.removeListener(_constants.eventTitles.HISTORY_POP_STATE, this._onpopstatecb);
        return;
      }

      if (typeof this._onpopstatecb === 'function') {
        this._events.removeListener(_constants.eventTitles.HISTORY_POP_STATE, this._onpopstatecb);
      }

      this._onpopstatecb = callback;
      this._events.on(_constants.eventTitles.HISTORY_POP_STATE, this._onpopstatecb);
    },
    get: function get() {
      return this._onpopstatecb;
    }
  }, {
    key: 'onbeforeunload',
    set: function set(callback) {
      if (typeof callback !== 'function') {
        this._events.removeListener(_constants.eventTitles.WINDOW_BEFORE_UNLOAD, this._onbeforeunloadecb);
        return;
      }

      if (typeof this._onbeforeunloadecb === 'function') {
        this._events.removeListener(_constants.eventTitles.WINDOW_BEFORE_UNLOAD, this._onbeforeunloadecb);
      }

      this._onbeforeunloadecb = callback;
      this._events.on(_constants.eventTitles.WINDOW_BEFORE_UNLOAD, this._onbeforeunloadecb);
    },
    get: function get() {
      return this._onbeforeunloadecb;
    }
  }]);
  return WindowStub;
}();

exports.default = WindowStub;